
module HdaAppPbaHelper

	def app_image(image)
	  File.join('hda_app_pba', image)
	end

	def sort_td_class_helper(param)
	  result = 'class="sortup"' if params[:sort] == param
	  result = 'class="sortdown"' if params[:sort] == param + "_reverse"
	  return result
	end

	def sort_link_helper(text, param)
	  key = param
	  key += "_reverse" if params[:sort] == param
	  options = {
	      :url => {:action => 'list', :params => params.merge({:sort => key, :page => nil})},
	      :update => 'table',
	      :before => "Element.show('spinner')",
	      :success => "Element.hide('spinner')"
	  }
	  html_options = {
	    :title => "Sort by this field",
	    :href => url_for(:action => 'list', :params => params.merge({:sort => key, :page => nil}))
	  }
	  link_to_remote(text, options, html_options)
	end

	def gen_icon(os, text, boot)
	  image_tag(app_image(os + ".png"), :align => 'absmiddle',
		:border=> 1, :alt => text, :title=> text,
		:class => (boot ? 'bootable' : 'not-bootable') )
	end

	def linux_icon(text, boot)
	  gen_icon('linux', text, boot)
	end

	def windows_icon(text, boot)
	  gen_icon('windows', text, boot)
	end

	def mac_icon(text, boot)
	  gen_icon('mac', text, boot)
	end

	def dos_icon(text, boot)
	  gen_icon('dos', text, boot)
	end

	def extended_icon(text, boot)
	  gen_icon('extended', text, boot)
	end

	def other_icon(text, boot)
	  gen_icon('other', text, boot)
	end

	def partition2icon(partition)
	  l = HdaAppPbaPartType.find(:first,
		:conditions => ["part_type = ?", partition.part_type])
	  label = l.nil? ? "unknown" : l.label
	  text = 'Id ' + partition.part_type.to_s(16) + ': ' +  label 
	  boot = partition.bootable
	  text += ", bootable" if boot
	  case label
	    when /linux/i: linux_icon(text, boot)
	    when /ntfs/i: windows_icon(text, boot)
	    when /win/i: windows_icon(text, boot)
	    # extended must be recognized before dos, otherwise
	    # it will be picked up as dos
	    when /extended/i: extended_icon(text, boot)
	    when /dos/i: dos_icon(text, boot)
	    when /mac/i: mac_icon(text, boot)
	    else other_icon(text, boot)
	  end
	end

	def partitions2display(partitions)
	  res = ""
	  begin
	    partitions.each do |p|
	      res += ' ' unless p == partitions.first
	      res += partition2icon(p)
	    end
	  # rescue
	  end
	  res
	end

	def partitions2display_old(partitions)
	  res = partitions.size.to_s
	  begin
	    res += ' ['
	    partitions.each do |p|
	      res += ', ' unless p == partitions.first
	      l = HdaAppPbaPartType.find(:first, :conditions => ["part_type = ?", p.part_type])
	      res += l.label
	    end
	    res += ']'
	  rescue
	    # prevent faults/exceptions
	  end
	  res
	end

	def pagination_links_remote(paginator)
	  page_options = {:window_size => 1}
	  pagination_links_each(paginator, page_options) do |n|
	    options = {
	      :url => {:action => 'list', :params => params.merge({:page => n})},
	      :update => 'table',
	      :before => "Element.show('spinner')",
	      :success => "Element.hide('spinner')"
	    }
	    html_options = {:href => url_for(:action => 'list', :params => params.merge({:page => n}))}
	    link_to_remote(n.to_s, options, html_options)
	  end
	end

end
