class HdaAppPbaBackup < ActiveRecord::Base

  has_many :hda_app_pba_partitions

  @@BASE = "/var/hda/apps/pba/backups"

  def remove_backup
    logger.warn("deleting backup " + self.full_name + "*!")
    FileUtils.cd(@@BASE) do
	FileUtils.rm(Dir.glob(self.full_name + "*"), :force => true)
    end
  end

  def full_name
    File.join("data", self.name + "_" + self.time.to_s)
  end

  def self.refresh_backups
	HdaAppPbaBackup.transaction do
	  HdaAppPbaPartition.transaction do
	    HdaAppPbaBackup.destroy_all
	    HdaAppPbaPartition.destroy_all
          end
	end
	base = @@BASE
	dir = File.join(base, "data")
	Dir.glob(File.join(dir, '*.part')) do |l|
	  if l =~ /\/([^\/]*)_([0-9]+)\.part$/
		name = $1
		time = $2.to_i
		# only create one if this backup entry is new!
		# assumes backups don't change once created!
		if HdaAppPbaBackup.find(:all, :conditions => ["name = ? and time = ?", name, time]).empty?
		  disk_size = backup_disk_size(dir, name, time)
		  (raw_size, partitions) = backup_raw_size(dir, name, time)
		  backup = HdaAppPbaBackup.new(:name => name, :time => time,
				:disk_size => disk_size,
				:raw_size => raw_size)
		  partitions.each do |t, size, boot|
			p = HdaAppPbaPartition.new(:part_type => t, :size => size, :bootable => boot)
			backup.hda_app_pba_partitions << p
		  end
		  backup.save
		end
	  end
	end
  end

private

  def self.backup_disk_size(dir, name, time)
    files = Dir.glob(File.join(dir, name) + '_' + time.to_s + '*')
    disk_size = 0
    files.each do |f|
      disk_size += File.stat(f).size
    end
    # FIXME: the second one is the raw size of the disk
    disk_size
  end

  def self.backup_raw_size(dir, name, time)
    file = File.join(dir, name) + '_' + time.to_s + '.part'
    partitions = IO.readlines(file).reject { |l| l !~ /size=/}
    sz = 0
    partitions.map! do |l|
      if l =~ /size=\s*(\d*),\s*Id=\s*([0-9a-fA-F]*)\s*(,\s*bootable)?/
	  size = $1.to_i * 512
	  part_type = $2.hex
	  bootable = ! $3.nil?
	  # do not double count extended partitions
	  sz += size unless is_extended?(part_type)
          [part_type, size, bootable]
      end
    end
    [sz, partitions.reject { |t, size, b| size == 0 } ]
  end

  # return true if a partition type is extended
  def self.is_extended? type
	[0xf, 0x85, 0x5].include? type
  end

end
