
class HdaAppPbaInstaller
  def up
    appname = "pba"
    app = "hda_app_" + appname
    tgt = "#{HDA_BASE}/platform/html"
    FileUtils.cd("#{HDA_BASE}/platform/apps/#{appname}") do
      FileUtils.cp("app/controllers/#{app}_controller.rb", "#{tgt}/app/controllers")
      FileUtils.cp("app/helpers/#{app}_helper.rb", "#{tgt}/app/helpers")
      FileUtils.cp_r(Dir.glob("app/models/#{app}*.rb"), "#{tgt}/app/models")
      FileUtils.mkdir_p("#{tgt}/app/views/#{app}")
      FileUtils.cp_r("app/views/#{app}", "#{tgt}/app/views/")
      FileUtils.mkdir_p("#{tgt}/public/images/#{app}")
      FileUtils.cp_r("public/images/.", "#{tgt}/public/images/#{app}/")
    end
    PbaTables.migrate :up
    pid = HdaPackage.find(:first, :conditions => "name='#{appname}'")[:id]
    HdaApplication.new(:hda_package_id => pid, :hda_user_id => 1).save
    Dir.glob("#{tgt}/app/models/#{app}*.rb").each { |f| load f }
    load "#{tgt}/app/controllers/#{app}_controller.rb"
    load "#{tgt}/app/helpers/#{app}_helper.rb"
    true
  end

  def down
    # FIXME - this could be simplified to only input the name once
    appname = "pba"
    app = "hda_app_" + appname
    tgt = "#{HDA_BASE}/platform/html"
    FileUtils.cd(tgt) do
      FileUtils.rm("app/controllers/#{app}_controller.rb", :force => true)
      FileUtils.rm("app/helpers/#{app}_helper.rb", :force => true)
      FileUtils.rm_rf(Dir.glob("app/models/#{app}*.rb"))
      FileUtils.rm_rf("app/views/#{app}")
      FileUtils.rm_rf("public/images/#{app}")
    end
    PbaTables.migrate :down
    pid = HdaPackage.find(:first, :conditions => "name='#{appname}'")[:id]
    HdaApplication.find(:all, :conditions => "hda_package_id=#{pid}").each {|r| r.destroy}
    #Object.__send__(:remove_const, 'HdaAppSlideshowController')
    #Object.__send__(:remove_const, 'HdaAppSlideshowPhoto')
    #Object.__send__(:remove_const, 'HdaAppSlideshowHelper')
    true
  end

  def user_up u
    true
  end

end

class PbaTables < ActiveRecord::Migration
  def self.up
    tablename = "hda_app_" + Inflector.pluralize("pba_backup")
    begin
        create_table tablename do |t|
          t.column :name, :string
          t.column :time, :integer
	  # the 8-byte limit requires a plugin called mysql_bignt
          t.column :disk_size, :integer, :limit => 8
          t.column :raw_size, :integer, :limit => 8
          t.column :notes, :text
        end
    rescue
      # do nothing - the db was already created
    end

    tablename = "hda_app_" + Inflector.pluralize("pba_partition")
    begin
        create_table tablename do |t|
          t.column :part_type, :integer
	  # the 8-byte limit requires a plugin called mysql_bignt
          t.column :size, :integer, :limit => 8
          t.column :bootable, :boolean
          t.column :hda_app_pba_backup_id, :integer
        end
    rescue
      # do nothing - the db was already created
    end

    tablename = "hda_app_" + Inflector.pluralize("pba_part_type")
    begin
        create_table tablename do |t|
          t.column :part_type, :integer
          t.column :label, :string
        end
	# add default partition IDs now
        self.add_part_ids
    rescue
      # do nothing - the db was already created
    end
  end

  def self.down
    # FIXME
    # dont' destroy data!
    # or ask the user if he/she wants to delete it!
  end

  def self.add_part_ids
	HdaAppPbaPartType.new(:part_type => '00'.hex, :label => 'Empty').save
	HdaAppPbaPartType.new(:part_type => '02'.hex, :label => 'XENIX root').save
	HdaAppPbaPartType.new(:part_type => '03'.hex, :label => 'XENIX /usr').save
	HdaAppPbaPartType.new(:part_type => '04'.hex, :label => 'DOS 3.0+ 16-bit FAT (up to 32M)').save
	HdaAppPbaPartType.new(:part_type => '05'.hex, :label => 'DOS 3.3+ Extended Partition').save
	HdaAppPbaPartType.new(:part_type => '06'.hex, :label => 'DOS 3.31+ 16-bit FAT (over 32M)').save
	HdaAppPbaPartType.new(:part_type => '07'.hex, :label => 'NTFS').save
	HdaAppPbaPartType.new(:part_type => '08'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '09'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '0a'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '0b'.hex, :label => 'WIN95 OSR2 FAT32').save
	HdaAppPbaPartType.new(:part_type => '0c'.hex, :label => 'WIN95 OSR2 FAT32, LBA').save
	HdaAppPbaPartType.new(:part_type => '0e'.hex, :label => 'WIN95: DOS 16-bit FAT, LBA').save
	HdaAppPbaPartType.new(:part_type => '0f'.hex, :label => 'WIN95: Extended partition, LBA').save
	HdaAppPbaPartType.new(:part_type => '10'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '11'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '12'.hex, :label => 'Configuration/diagnostics partition').save
	HdaAppPbaPartType.new(:part_type => '14'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '16'.hex, :label => 'Hidden DOS 16-bit FAT').save
	HdaAppPbaPartType.new(:part_type => '17'.hex, :label => 'Hidden IFS').save
	HdaAppPbaPartType.new(:part_type => '18'.hex, :label => 'AST SmartSleep Partition').save
	HdaAppPbaPartType.new(:part_type => '19'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '1b'.hex, :label => 'Hidden WIN95 OSR2 FAT32').save
	HdaAppPbaPartType.new(:part_type => '1c'.hex, :label => 'Hidden WIN95 OSR2 FAT32, LBA').save
	HdaAppPbaPartType.new(:part_type => '1e'.hex, :label => 'Hidden WIN95 16-bit FAT, LBA').save
	HdaAppPbaPartType.new(:part_type => '20'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '21'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '22'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '23'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '24'.hex, :label => 'NEC DOS 3.x').save
	HdaAppPbaPartType.new(:part_type => '26'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '2a'.hex, :label => 'AtheOS File System (AFS)').save
	HdaAppPbaPartType.new(:part_type => '2b'.hex, :label => 'SyllableSecure (SylStor)').save
	HdaAppPbaPartType.new(:part_type => '31'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '32'.hex, :label => 'NOS').save
	HdaAppPbaPartType.new(:part_type => '33'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '34'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '35'.hex, :label => 'JFS on OS/2 or eCS').save
	HdaAppPbaPartType.new(:part_type => '38'.hex, :label => 'THEOS ver 3.2 2gb partition').save
	HdaAppPbaPartType.new(:part_type => '39'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '3a'.hex, :label => 'THEOS ver 4 4gb partition').save
	HdaAppPbaPartType.new(:part_type => '3b'.hex, :label => 'THEOS ver 4 extended partition').save
	HdaAppPbaPartType.new(:part_type => '3d'.hex, :label => 'Hidden NetWare').save
	HdaAppPbaPartType.new(:part_type => '40'.hex, :label => 'Venix 80286').save
	HdaAppPbaPartType.new(:part_type => '41'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '42'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '43'.hex, :label => 'Linux native (sharing disk with DRDOS)').save
	HdaAppPbaPartType.new(:part_type => '44'.hex, :label => 'GoBack partition').save
	HdaAppPbaPartType.new(:part_type => '45'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '46'.hex, :label => 'EUMEL/Elan').save
	HdaAppPbaPartType.new(:part_type => '47'.hex, :label => 'EUMEL/Elan').save
	HdaAppPbaPartType.new(:part_type => '48'.hex, :label => 'EUMEL/Elan').save
	HdaAppPbaPartType.new(:part_type => '4a'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '4c'.hex, :label => 'Oberon partition').save
	HdaAppPbaPartType.new(:part_type => '4d'.hex, :label => 'QNX4.x').save
	HdaAppPbaPartType.new(:part_type => '4e'.hex, :label => 'QNX4.x 2nd part').save
	HdaAppPbaPartType.new(:part_type => '4f'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '50'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '51'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '52'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '53'.hex, :label => 'Disk Manager 6.0 Aux3').save
	HdaAppPbaPartType.new(:part_type => '54'.hex, :label => 'Disk Manager 6.0 Dynamic Drive Overlay (DDO)').save
	HdaAppPbaPartType.new(:part_type => '55'.hex, :label => 'EZ-Drive').save
	HdaAppPbaPartType.new(:part_type => '56'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '56'.hex, :label => 'AT&T MS-DOS 3.x logically sectored FAT.').save
	HdaAppPbaPartType.new(:part_type => '57'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '5c'.hex, :label => 'Priam EDisk').save
	HdaAppPbaPartType.new(:part_type => '61'.hex, :label => 'SpeedStor').save
	HdaAppPbaPartType.new(:part_type => '63'.hex, :label => 'Unix System V (SCO, ISC Unix, UnixWare, ...), Mach, GNU Hurd').save
	HdaAppPbaPartType.new(:part_type => '64'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '65'.hex, :label => 'Novell Netware 386, 3.xx or 4.xx').save
	HdaAppPbaPartType.new(:part_type => '66'.hex, :label => 'Novell Netware SMS Partition').save
	HdaAppPbaPartType.new(:part_type => '67'.hex, :label => 'Novell').save
	HdaAppPbaPartType.new(:part_type => '68'.hex, :label => 'Novell').save
	HdaAppPbaPartType.new(:part_type => '69'.hex, :label => 'Novell Netware 5+, Novell Netware NSS Partition').save
	HdaAppPbaPartType.new(:part_type => '6e'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '70'.hex, :label => 'DiskSecure Multi-Boot').save
	HdaAppPbaPartType.new(:part_type => '71'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '73'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '74'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '75'.hex, :label => 'IBM PC/IX').save
	HdaAppPbaPartType.new(:part_type => '76'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '77'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '78'.hex, :label => 'XOSL FS').save
	HdaAppPbaPartType.new(:part_type => '7e'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '7f'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '80'.hex, :label => 'MINIX until 1.4a').save
	HdaAppPbaPartType.new(:part_type => '81'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '82'.hex, :label => 'Linux swap').save
	HdaAppPbaPartType.new(:part_type => '83'.hex, :label => 'Linux native partition').save
	HdaAppPbaPartType.new(:part_type => '84'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '85'.hex, :label => 'Linux extended partition').save
	HdaAppPbaPartType.new(:part_type => '86'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '87'.hex, :label => 'NTFS volume set').save
	HdaAppPbaPartType.new(:part_type => '88'.hex, :label => 'Linux plaintext partition table').save
	HdaAppPbaPartType.new(:part_type => '8a'.hex, :label => 'Linux Kernel Partition (used by AiR-BOOT)').save
	HdaAppPbaPartType.new(:part_type => '8b'.hex, :label => 'Legacy Fault Tolerant FAT32 volume').save
	HdaAppPbaPartType.new(:part_type => '8c'.hex, :label => 'Legacy Fault Tolerant FAT32 volume using BIOS extd INT 13h').save
	HdaAppPbaPartType.new(:part_type => '8d'.hex, :label => 'Free FDISK hidden Primary DOS FAT12 partitition').save
	HdaAppPbaPartType.new(:part_type => '8e'.hex, :label => 'Linux Logical Volume Manager partition').save
	HdaAppPbaPartType.new(:part_type => '90'.hex, :label => 'Free FDISK hidden Primary DOS FAT16 partitition').save
	HdaAppPbaPartType.new(:part_type => '91'.hex, :label => 'Free FDISK hidden DOS extended partitition').save
	HdaAppPbaPartType.new(:part_type => '92'.hex, :label => 'Free FDISK hidden Primary DOS large FAT16 partitition').save
	HdaAppPbaPartType.new(:part_type => '93'.hex, :label => 'Hidden Linux native partition').save
	HdaAppPbaPartType.new(:part_type => '94'.hex, :label => 'Amoeba bad block table').save
	HdaAppPbaPartType.new(:part_type => '95'.hex, :label => 'MIT EXOPC native partitions').save
	HdaAppPbaPartType.new(:part_type => '97'.hex, :label => 'Free FDISK hidden Primary DOS FAT32 partitition').save
	HdaAppPbaPartType.new(:part_type => '98'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => '99'.hex, :label => 'DCE376 logical drive').save
	HdaAppPbaPartType.new(:part_type => '9a'.hex, :label => 'Free FDISK hidden Primary DOS FAT16 partitition (LBA)').save
	HdaAppPbaPartType.new(:part_type => '9b'.hex, :label => 'Free FDISK hidden DOS extended partitition (LBA)').save
	HdaAppPbaPartType.new(:part_type => '9f'.hex, :label => 'BSD/OS').save
	HdaAppPbaPartType.new(:part_type => 'a1'.hex, :label => 'HP Volume Expansion (SpeedStor variant)').save
	HdaAppPbaPartType.new(:part_type => 'a3'.hex, :label => 'HP Volume Expansion (SpeedStor variant)').save
	HdaAppPbaPartType.new(:part_type => 'a4'.hex, :label => 'HP Volume Expansion (SpeedStor variant)').save
	HdaAppPbaPartType.new(:part_type => 'a5'.hex, :label => 'BSD/386, 386BSD, NetBSD, FreeBSD').save
	HdaAppPbaPartType.new(:part_type => 'a6'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => 'a7'.hex, :label => 'NeXTStep').save
	HdaAppPbaPartType.new(:part_type => 'a8'.hex, :label => 'Mac OS-X').save
	HdaAppPbaPartType.new(:part_type => 'a9'.hex, :label => 'NetBSD').save
	HdaAppPbaPartType.new(:part_type => 'aa'.hex, :label => 'Olivetti Fat 12 1.44MB Service Partition').save
	HdaAppPbaPartType.new(:part_type => 'ab'.hex, :label => 'Mac OS-X Boot partition').save
	HdaAppPbaPartType.new(:part_type => 'ae'.hex, :label => 'ShagOS filesystem').save
	HdaAppPbaPartType.new(:part_type => 'af'.hex, :label => 'MacOS X HFS/HFS+').save
	HdaAppPbaPartType.new(:part_type => 'b0'.hex, :label => 'BootStar Dummy').save
	HdaAppPbaPartType.new(:part_type => 'b1'.hex, :label => 'HP Volume Expansion (SpeedStor variant)').save
	HdaAppPbaPartType.new(:part_type => 'b3'.hex, :label => 'HP Volume Expansion (SpeedStor variant)').save
	HdaAppPbaPartType.new(:part_type => 'b4'.hex, :label => 'HP Volume Expansion (SpeedStor variant)').save
	HdaAppPbaPartType.new(:part_type => 'b6'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => 'b7'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => 'b8'.hex, :label => 'BSDI BSD/386 swap partition').save
	HdaAppPbaPartType.new(:part_type => 'bb'.hex, :label => 'Boot Wizard hidden').save
	HdaAppPbaPartType.new(:part_type => 'c0'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => 'c1'.hex, :label => 'DRDOS/secured (FAT-12)').save
	HdaAppPbaPartType.new(:part_type => 'c2'.hex, :label => 'Hidden Linux').save
	HdaAppPbaPartType.new(:part_type => 'c3'.hex, :label => 'Hidden Linux swap').save
	HdaAppPbaPartType.new(:part_type => 'c4'.hex, :label => 'DRDOS/secured (FAT-16, < 32M)').save
	HdaAppPbaPartType.new(:part_type => 'c5'.hex, :label => 'DRDOS/secured (extended)').save
	HdaAppPbaPartType.new(:part_type => 'c6'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => 'c7'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => 'c8'.hex, :label => 'Unknown for DR-DOS 8.0+').save
	HdaAppPbaPartType.new(:part_type => 'c9'.hex, :label => 'Unknown for DR-DOS 8.0+').save
	HdaAppPbaPartType.new(:part_type => 'ca'.hex, :label => 'Unknown for DR-DOS 8.0+').save
	HdaAppPbaPartType.new(:part_type => 'cb'.hex, :label => 'DR-DOS 7.04+ secured FAT32 (CHS)').save
	HdaAppPbaPartType.new(:part_type => 'cc'.hex, :label => 'DR-DOS 7.04+ secured FAT32 (LBA)').save
	HdaAppPbaPartType.new(:part_type => 'cd'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => 'ce'.hex, :label => 'DR-DOS 7.04+ FAT16X (LBA)').save
	HdaAppPbaPartType.new(:part_type => 'cf'.hex, :label => 'DR-DOS 7.04+ secured EXT DOS (LBA)').save
	HdaAppPbaPartType.new(:part_type => 'd0'.hex, :label => 'REAL/32 secure big partition').save
	HdaAppPbaPartType.new(:part_type => 'd1'.hex, :label => 'Old Multiuser DOS secured FAT12').save
	HdaAppPbaPartType.new(:part_type => 'd4'.hex, :label => 'Old Multiuser DOS secured FAT16 <32M').save
	HdaAppPbaPartType.new(:part_type => 'd5'.hex, :label => 'Old Multiuser DOS secured extended partition').save
	HdaAppPbaPartType.new(:part_type => 'd6'.hex, :label => 'Old Multiuser DOS secured FAT16 >=32M').save
	HdaAppPbaPartType.new(:part_type => 'd8'.hex, :label => 'CP/M-86').save
	HdaAppPbaPartType.new(:part_type => 'da'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => 'db'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => 'dd'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => 'de'.hex, :label => 'Dell PowerEdge Server utilities (FAT fs)').save
	HdaAppPbaPartType.new(:part_type => 'df'.hex, :label => 'DG/UX virtual disk manager partition').save
	HdaAppPbaPartType.new(:part_type => 'e0'.hex, :label => 'Unknown by STMicroelectronics for a filesystem called ST AVFS.').save
	HdaAppPbaPartType.new(:part_type => 'e1'.hex, :label => 'DOS access or SpeedStor 12-bit FAT extended partition').save
	HdaAppPbaPartType.new(:part_type => 'e3'.hex, :label => 'DOS R/O or SpeedStor').save
	HdaAppPbaPartType.new(:part_type => 'e4'.hex, :label => 'SpeedStor 16-bit FAT extended partition < 1024 cyl.').save
	HdaAppPbaPartType.new(:part_type => 'e5'.hex, :label => 'Tandy MSDOS with logically sectored FAT').save
	HdaAppPbaPartType.new(:part_type => 'e6'.hex, :label => 'Storage Dimensions SpeedStor').save
	HdaAppPbaPartType.new(:part_type => 'eb'.hex, :label => 'BeOS BFS').save
	HdaAppPbaPartType.new(:part_type => 'ec'.hex, :label => 'SkyOS SkyFS').save
	HdaAppPbaPartType.new(:part_type => 'f0'.hex, :label => 'Linux/PA-RISC boot loader').save
	HdaAppPbaPartType.new(:part_type => 'f1'.hex, :label => 'Storage Dimensions SpeedStor').save
	HdaAppPbaPartType.new(:part_type => 'f2'.hex, :label => 'DOS 3.3+ secondary partition').save
	HdaAppPbaPartType.new(:part_type => 'f3'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => 'f4'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => 'f5'.hex, :label => 'Prologue multi-volume partition').save
	HdaAppPbaPartType.new(:part_type => 'f6'.hex, :label => 'Storage Dimensions SpeedStor').save
	HdaAppPbaPartType.new(:part_type => 'f7'.hex, :label => 'Unknown').save
	HdaAppPbaPartType.new(:part_type => 'f9'.hex, :label => 'pCache').save
	HdaAppPbaPartType.new(:part_type => 'fa'.hex, :label => 'Bochs').save
	HdaAppPbaPartType.new(:part_type => 'fb'.hex, :label => 'VMware File System partition').save
	HdaAppPbaPartType.new(:part_type => 'fc'.hex, :label => 'VMware Swap partition').save
	HdaAppPbaPartType.new(:part_type => 'fd'.hex, :label => 'Linux raid partition with autodetect using persistent superblock').save
	HdaAppPbaPartType.new(:part_type => 'fe'.hex, :label => 'Unknown').save
  end
end

